package ContentFieldName::Tags;

use strict;
use warnings;
use Data::Dumper;

sub _hdlr_content_field_name {
    my ( $ctx, $args, $cond ) = @_;

    my $content_type = $ctx->stash('content_type')
        or return $ctx->_no_content_type_error;
    my $content_data = $ctx->stash('content')
        or return $ctx->_no_content_error;
    my $field_data = $ctx->stash('content_field_data')
        or return $ctx->_no_content_field_error;

    my $field_type = $ctx->stash('content_field_type')
        || MT->registry('content_field_types')->{ $field_data->{type} };
    return $ctx->error(
        MT->translate('No Content Field Type could be found.') )
        unless $field_type;

    # __key__:`FbN{bNX/ZNg{bNX/WI{^
    # __key__Ȃ:^O/JeS/Rec^Cv
    my $value;
    if (( $field_data->{type} eq 'select_box' ) ||
        ( $field_data->{type} eq 'radio_button' ) ||
        ( $field_data->{type} eq 'checkboxes' )) {
        $value = $ctx->{__stash}{vars}{__key__};
    } elsif ( $field_data->{type} eq 'tags' ) {
        $value = field_name_handler( $ctx, $args, $cond, $field_data, $value );
    } elsif ( $field_data->{type} eq 'categories' ) {
        $value = _hdlr_category_label( $ctx, $args, $cond );
    } elsif ( $field_data->{type} eq 'content_type' ) {
        $value = _hdlr_content_field_value( $ctx, $args, $cond );
    # single_line_text
    # multi_line_text
    # number
    # url
    # date_and_time
    # date_only
    # time_only
    # asset
    # asset_audio
    # asset_video
    # asset_image
    # embedded_text
    # list
    # tables
    } else {
        $value = $ctx->{__stash}{vars}{__value__};
    }
    return $value;
}
sub field_name_handler {
    my ( $ctx, $args, $cond, $field_data, $value ) = @_;
    my $content = $ctx->stash('Tag');
    return $content ? $content->name : $value;
}

# MT::Template::Tags::Category
sub _hdlr_category_label {
    my ( $ctx, $args, $cond ) = @_;

    # Get the current category
    defined( my $cat = _get_category_context($ctx) )
        or return (
        defined( $args->{default} )
        ? $args->{default}
        : $ctx->error( $ctx->errstr )
        );

    return ( defined( $args->{default} ) ? $args->{default} : '' )
        if ( $cat eq '' );

    my $label = $cat->label;
    $label = '' unless defined $label;
    return $label;
}

sub _get_category_context {
    my ($ctx) = @_;

    my $tag = $ctx->stash('tag');

    # Get our hands on the category for the current context
    # Either in MTCategories, a Category Archive Template
    # Or the category for the current entry
    my $cat = $ctx->stash('category')
        || $ctx->stash('archive_category');

    if ( !defined $cat ) {

        # No category found so far, test the entry
        if ( $ctx->stash('entry') ) {
            $cat = $ctx->stash('entry')->category;

            # Return empty string if entry has no category
            # as the tag has been used in the correct context
            # but there is no category to work with
            return '' if ( !defined $cat );
        }
        elsif ( my $content_data = $ctx->stash('content') ) {
            my $map = $ctx->stash('template_map');
            unless ($map) {
                my $content_type_id = $content_data->content_type_id;
                ($map) = MT->model('templatemap')->load(
                    { archive_type => 'ContentType', is_preferred => 1 },
                    {   join => MT->model('template')->join_on(
                            undef,
                            {   id => \'= templatemap_template_id',
                                content_type_id => $content_type_id,
                            },
                        )
                    }
                );
            }
            return '' if ( !$map || !$map->cat_field_id );
            my ($objectcategory) = MT->model('objectcategory')->load(
                {   object_ds  => 'content_data',
                    object_id  => $content_data->content_type_id,
                    cf_id      => $map->cat_field_id,
                    is_primary => 1,
                }
            );
            return '' unless ($objectcategory);
            $cat
                = MT->model('category')->load( $objectcategory->category_id );
        }
        else {
            return $ctx->error(
                MT->translate(
                    "MT[_1] must be used in a [_2] context",
                    $tag,
                    $tag =~ m/folder/ig ? 'folder' : 'category'
                )
            );
        }
    }
    return $cat;
}

# MT::Template::Tags::ContentType
sub _hdlr_content_field_value {
    my ( $ctx, $args, $cond ) = @_;

    my $content_type = $ctx->stash('content_type')
        or return $ctx->_no_content_type_error;
    my $content_data = $ctx->stash('content')
        or return $ctx->_no_content_error;
    my $field_data = $ctx->stash('content_field_data')
        or return $ctx->_no_content_field_error;

    my $field_type = $ctx->stash('content_field_type')
        || MT->registry('content_field_types')->{ $field_data->{type} };
    return $ctx->error(
        MT->translate('No Content Field Type could be found.') )
        unless $field_type;

    my $value = $ctx->{__stash}{vars}{__value__};
    if ( my $handler = $field_type->{field_value_handler} ) {
        if ( !ref $handler ) {
            $handler = MT->handler_to_coderef($handler);
        }
        return $ctx->error(
            MT->translate(
                'Invalid field_value_handler of [_1].',
                $field_data->{type}
            )
        ) unless ref $handler eq 'CODE';
        $value = $handler->( $ctx, $args, $cond, $field_data, $value );
    }

    return $value;
}

1;
